'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (t, node, tagName) {
  function buildProps(node, defaultCss, cssProps) {
    var css = buildDefaultCssProp(t, defaultCss);
    var cssTemplate = css.value.expression;
    var props = [css];

    if (node.openingElement.attributes == null) {
      return props;
    }

    node.openingElement.attributes.forEach(function (attribute) {
      var name = attribute.name.name;

      if (name in propsToOmit) {
        return;
      } else if (name === 'css') {
        addTemplateToTemplate(cssTemplate, attribute.value.expression);
      } else if (name in cssProps) {
        addCssProp(cssTemplate, attribute, cssProps[name]);
      } else if (name in booleanProps) {
        addBooleanProp(cssTemplate, attribute, name, booleanProps[name]);
      } else if (name === 'grow') {
        addGrowProp(cssTemplate, attribute);
      } else {
        props.push(attribute);
      }
    });

    return props;
  }

  function addCssProp(cssTemplate, attribute, name) {
    var value = attribute.value;


    if (t.isJSXExpressionContainer(value)) {
      var expression = value.expression;

      // console.log(printAST(expression));

      if (t.isNumericLiteral(expression)) {
        addStringToTemplate(cssTemplate, name + ': ' + expression.extra.raw + ';');
      } else if (t.isStringLiteral(expression)) {
        addStringToTemplate(cssTemplate, name + ': ' + expression.value + ';');
      } else if (t.isIdentifier(expression)) {
        addStringToTemplate(cssTemplate, name + ': ');
        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: ';', cooked: ';' }));
        addExpressionToTemplate(cssTemplate, t.identifier(expression.name));
      } else if (t.isTemplateLiteral(expression)) {
        expression.quasis[0].value.cooked = name + ': ' + expression.quasis[0].value.cooked;
        expression.quasis[0].value.raw = name + ': ' + expression.quasis[0].value.raw;
        addTemplateToTemplate(cssTemplate, expression);
        addStringToTemplate(cssTemplate, ';');
      } else if (t.isBinaryExpression(expression)) {
        addStringToTemplate(cssTemplate, name + ': ');
        addExpressionToTemplate(cssTemplate, expression);
        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: ';', cooked: ';' }));
      } else if (t.isConditionalExpression(expression)) {
        addStringToTemplate(cssTemplate, name + ': ');
        addExpressionToTemplate(cssTemplate, expression);
        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: ';', cooked: ';' }));
      }
    } else if (t.isStringLiteral(value)) {
      addStringToTemplate(cssTemplate, name + ': ' + value.value + ';');
    }
  }

  function addBooleanProp(cssTemplate, attribute, name, _ref) {
    var consequent = _ref.consequent,
        alternate = _ref.alternate;
    var value = attribute.value;


    if (value === null) {
      addStringToTemplate(cssTemplate, consequent);
    } else if (t.isJSXExpressionContainer(value)) {
      var expression = value.expression;


      if (t.isBooleanLiteral(expression) && expression.value === true) {
        addStringToTemplate(cssTemplate, consequent);
      } else if (t.isIdentifier(expression)) {
        addExpressionToTemplate(cssTemplate, t.conditionalExpression(t.binaryExpression('===', t.identifier(expression.name), t.booleanLiteral(true)), t.stringLiteral(consequent), t.stringLiteral(alternate)));

        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: '', cooked: '' }));
      }
    }
  }

  function addGrowProp(cssTemplate, attribute) {
    var value = attribute.value;


    if (value === null) {
      addStringToTemplate(cssTemplate, 'flex-grow: 1;');
    } else if (t.isJSXExpressionContainer(value)) {
      var expression = value.expression;


      if (t.isNumericLiteral(expression)) {
        addStringToTemplate(cssTemplate, 'flex-grow: ' + expression.extra.raw + ';');
      } else if (t.isStringLiteral(expression)) {
        addStringToTemplate(cssTemplate, 'flex-grow: ' + expression.value + ';');
      } else if (t.isIdentifier(expression)) {
        addStringToTemplate(cssTemplate, 'flex-grow: ');
        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: ';', cooked: ';' }));
        addExpressionToTemplate(cssTemplate, t.identifier(expression.name));
      }
    } else if (t.isStringLiteral(value)) {
      addStringToTemplate(cssTemplate, 'flex-grow: ' + value.value + ';');
    }
  }

  if (tagName === 'view') {
    renameTag(node);
    node.openingElement.attributes = buildProps(node, defaultColCss, propsToUse);
  } else if (tagName === 'col') {
    renameTag(node);
    node.openingElement.attributes = buildProps(node, defaultColCss, propsToUse);
  } else if (tagName === 'row') {
    renameTag(node);
    node.openingElement.attributes = buildProps(node, defaultRowCss, propsToUse);
  } else if (tagName === 'flex') {
    renameTag(node);
    node.openingElement.attributes = buildProps(node, defaultFlexCss, flexPropsToUse);
  }
};

// const printAST = require('ast-pretty-print')
var _require = require('./utils'),
    addTemplateToTemplate = _require.addTemplateToTemplate,
    addStringToTemplate = _require.addStringToTemplate,
    addQuasiToTemplate = _require.addQuasiToTemplate,
    addExpressionToTemplate = _require.addExpressionToTemplate,
    buildDefaultCssProp = _require.buildDefaultCssProp,
    renameTag = _require.renameTag;

var propsToOmit = {
  as: true
};

var propsToUse = {
  top: 'top',
  right: 'right',
  bottom: 'bottom',
  left: 'left',
  width: 'width',
  maxWidth: 'max-width',
  minWidth: 'min-width',
  height: 'height',
  maxHeight: 'max-height',
  minHeight: 'min-height',

  flex: 'flex',
  wrap: 'flex-wrap',
  // grow: 'flex-grow',
  shrink: 'flex-shrink',
  basis: 'flex-basis',
  order: 'order',
  alignContent: 'align-content',
  alignSelf: 'align-self',
  align: 'align-items',
  justify: 'justify-content',
  // alignVertical
  // alignHorizontal

  padding: 'padding',
  paddingTop: 'padding-top',
  paddingRight: 'padding-right',
  paddingBottom: 'padding-bottom',
  paddingLeft: 'padding-left',
  margin: 'margin',
  marginTop: 'margin-top',
  marginRight: 'margin-right',
  marginBottom: 'margin-bottom',
  marginLeft: 'margin-left',

  position: 'position',
  overflow: 'overflow',
  overflowX: 'overflow-x',
  overflowY: 'overflow-y',
  //WebkitOverflowScrolling
  zIndex: 'z-index'
};

var flexPropsToUse = _extends({}, propsToUse, {
  direction: 'flex-direction'
});

var booleanProps = {
  center: {
    consequent: 'align-items: center;justify-content: center;',
    alternate: ''
  },
  hidden: {
    consequent: 'display: none;',
    alternate: ''
  },
  inline: {
    consequent: 'display: inline-flex;',
    alternate: ''
  },
  fit: {
    consequent: 'height: 100%;width: 100%;',
    alternate: ''
  },
  absoluteFill: {
    consequent: 'position: absolute;top: 0;right: 0;bottom: 0;left: 0;',
    alternate: ''
  }
};

var defaultFlexCss = 'display: flex;flex-shrink: 0;align-content: flex-start;position: relative;';
var defaultColCss = 'display: flex;flex-direction: column;flex-shrink: 0;align-content: flex-start;position: relative;';
var defaultRowCss = 'display: flex;flex-direction: row;flex-shrink: 0;align-content: flex-start;position: relative;';