'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (t, node) {
  function buildProps(node) {
    var css = buildDefaultCssProp(t, defaultCss);
    var cssTemplate = css.value.expression;
    var props = [css];

    if (node.openingElement.attributes == null) {
      return props;
    }

    node.openingElement.attributes.forEach(function (attribute) {
      var name = attribute.name.name;

      if (name in propsToOmit) {
        return;
      } else if (name === 'css') {
        addTemplateToTemplate(cssTemplate, attribute.value.expression);
      } else if (name in propsToUse) {
        addCssProp(cssTemplate, attribute, propsToUse[name]);
      } else {
        props.push(attribute);
      }
    });

    return props;
  }

  function addCssProp(cssTemplate, attribute, name) {
    var value = attribute.value;


    if (t.isJSXExpressionContainer(value)) {
      var expression = value.expression;

      // console.log(printAST(expression));

      if (t.isStringLiteral(expression)) {
        addStringToTemplate(cssTemplate, name + ': ' + expression.value + ';');
      } else if (t.isIdentifier(expression)) {
        addStringToTemplate(cssTemplate, name + ': ');
        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: ';', cooked: ';' }));
        addExpressionToTemplate(cssTemplate, t.identifier(expression.name));
      } else if (t.isTemplateLiteral(expression)) {
        expression.quasis[0].value.cooked = name + ': ' + expression.quasis[0].value.cooked;
        expression.quasis[0].value.raw = name + ': ' + expression.quasis[0].value.raw;
        addTemplateToTemplate(cssTemplate, expression);
        addStringToTemplate(cssTemplate, ';');
      } else if (t.isConditionalExpression(expression)) {
        addStringToTemplate(cssTemplate, name + ': ');
        addExpressionToTemplate(cssTemplate, expression);
        addQuasiToTemplate(cssTemplate, t.templateElement({ raw: ';', cooked: ';' }));
      }
    } else if (t.isStringLiteral(value)) {
      addStringToTemplate(cssTemplate, name + ': ' + value.value + ';');
    }
  }

  renameTag(node);
  node.openingElement.attributes = buildProps(node);
};

// const printAST = require('ast-pretty-print')
var _require = require('./utils'),
    addTemplateToTemplate = _require.addTemplateToTemplate,
    addStringToTemplate = _require.addStringToTemplate,
    addQuasiToTemplate = _require.addQuasiToTemplate,
    addExpressionToTemplate = _require.addExpressionToTemplate,
    buildDefaultCssProp = _require.buildDefaultCssProp,
    renameTag = _require.renameTag;

var propsToOmit = {
  as: true
};

var propsToUse = {
  size: 'flex-basis'
};

var defaultCss = 'flex-grow: 0;flex-shrink: 0;';